import java.util.*;
import java.io.*;

public class MakePegHiQRule {

	int startingConfiguration[][] = {
		{ 0,  0,  0,  1,  0,  0,  0},
		{ 0,  0,  1,  1,  1,  0,  0},
		{ 0,  1,  1,  1,  1,  1,  0},
		{ 1,  1,  1,  0,  1,  1,  1},
		{ 0,  1,  1,  1,  1,  1,  0},
		{ 0,  0,  1,  1,  1,  0,  0},
		{ 0,  0,  0,  1,  0,  0,  0}
	};

	public final static int MAX_BLOCKS = 33;
	public final static int ENDING_PEG = 17;
	public static int MAX_MOVES;

	int dr[] = {1, 0};
	int dc[] = {0, 1};
	int labels[][] = {
		{ 0,  0,  1,  2,  3,  0,  0},
		{ 0,  0,  4,  5,  6,  0,  0},
		{ 7,  8,  9, 10, 11, 12, 13},
		{14, 15, 16, 17, 18, 19, 20},
		{21, 22, 23, 24, 25, 26, 27},
		{ 0,  0, 28, 29, 30,  0,  0},
		{ 0,  0, 31, 32, 33,  0,  0}
	};

	ArrayList expression;
	ArrayList output = new ArrayList();
	int legal[][];

	boolean valid (int r, int c){ return r >= 0 && r < 7 && c >= 0 && c < 7; }
	void findTransitions (){
		ArrayList transitions = new ArrayList();
		for (int r = 0; r < 7; r++){
			for (int c = 0; c < 7; c++) if (labels[r][c] > 0){
				for (int d = 0; d < 2; d++) if (valid (r + 2 * dr[d], c + 2 * dc[d])){
					if (labels[r + 2 * dr[d]][c + 2 * dc[d]] > 0){
						int[] record = new int[3];
						record[0] = labels[r][c];
						record[1] = labels[r + dr[d]][c + dc[d]];
						record[2] = labels[r + 2 * dr[d]][c + 2 * dc[d]];
						transitions.add (record);
					}
				}
			}
		}
		legal = (int[][]) transitions.toArray (new int[0][0]);
	}

	String getBlock (int block, int moveNum){
		return "" + ((moveNum - 1) * MAX_BLOCKS + block) /*+ " (" + block + "," + moveNum + ")"*/;
 	}

	String getRule (int index, int moveNum){
		return "" + (MAX_MOVES * MAX_BLOCKS + index + 1 + (moveNum - 2) * legal.length) /*+ " [" + index + "," + moveNum + "]"*/;
	}

	void makeTransitions(){
		// Starting position.
		for (int a = 0; a < 7; a++){
			for (int b = 0; b < 7; b++){
				if (labels[a][b] > 0){
					if (startingConfiguration[a][b] > 0){
						output.add (getBlock (labels[a][b], 1) + " 0");
						MAX_MOVES++;
					}
					else
						output.add ("-" + getBlock (labels[a][b], 1) + " 0");
				}
			}
		}

		// Ending position.
		for (int a = 1; a <= MAX_BLOCKS; a++)
			if (a == ENDING_PEG) output.add (getBlock (a, MAX_MOVES) + " 0");
			else output.add ("-" + getBlock (a, MAX_MOVES) + " 0");

		// Create in-between position transitions...
		for (int moves = 2; moves <= MAX_MOVES; moves++){

			boolean defined[] = new boolean[legal.length];
			boolean check[][] = new boolean[legal.length][legal.length];
			for (int a = 0; a < legal.length; a++) check[a][a] = true;

			// No change checking unless rule applied...
			for (int a = 1; a <= MAX_BLOCKS; a++){
				String previmpliescurr = "-" + getBlock(a, moves - 1) + " " + getBlock(a, moves);
				String currimpliesprev = "" + getBlock(a, moves - 1) + " -" + getBlock(a, moves);
				String changecond = "";
				for (int b = 0; b < legal.length; b++){
					if (a == legal[b][0] || a == legal[b][1] || a == legal[b][2]){
						
						
						// Define a new rule...
						if (!defined[b]){
							defined[b] = true;
							String ruleName = "-" + getRule (b, moves) + " ";
							output.add (ruleName + getBlock (legal[b][1], moves - 1) + " 0");
							output.add (ruleName + "-" + getBlock (legal[b][1], moves) + " 0");
							output.add (ruleName + getBlock (legal[b][0], moves - 1) + " " + getBlock (legal[b][2], moves - 1) + " 0");
							output.add (ruleName + "-" + getBlock (legal[b][0], moves - 1) + " -" + getBlock (legal[b][2], moves - 1) + " 0");
							output.add (ruleName + getBlock (legal[b][0], moves - 1) + " " + getBlock (legal[b][0], moves) + " 0");
							output.add (ruleName + "-" + getBlock (legal[b][0], moves - 1) + " -" + getBlock (legal[b][0], moves) + " 0");
							output.add (ruleName + getBlock (legal[b][2], moves - 1) + " " + getBlock (legal[b][2], moves) + " 0");
							output.add (ruleName + "-" + getBlock (legal[b][2], moves - 1) + " -" + getBlock (legal[b][2], moves) + " 0");	
							// Make sure no other rules are being used.
							for (int c = 0; c < legal.length; c++){
								if (!check[b][c] && defined[c]){
									check[b][c] = check[c][b] = true;
									output.add ("-" + getRule (b, moves) + " -" + getRule (c, moves) + " 0");
								}
							}
							
						}
						
						changecond += " " + getRule (b, moves);
					}
				}
				output.add (previmpliescurr + changecond + " 0");
				output.add (currimpliesprev + changecond + " 0");
			}

			// Pick a rule.
			String t = "";
			for (int a = 0; a < legal.length; a++)
				t += getRule (a, moves) + " ";
			output.add (t + "0");

		}

		System.out.println ("p cnf " + getRule (legal.length - 1, MAX_MOVES) + " " + output.size());
		for (int a = 0; a < output.size(); a++)
			System.out.println (output.get (a));
	}

	void run(){
		findTransitions();
		makeTransitions();
	}

	public static void main (String args[]){
		(new MakePegHiQRule()).run();
	}
}
