import java.util.*;
import java.io.*;

public class MakePegTriangleRule {
	int legal[][] = {
		{ 1, 2, 4}, { 1, 3, 6}, { 2, 4, 7}, { 2, 5, 9}, { 3, 5, 8}, { 3, 6,10},
		{ 4, 5, 6}, { 4, 8,13}, { 4, 7,11}, { 5, 8,12}, { 5, 9,14}, { 6, 9,13},
		{ 6,10,15}, { 7, 8, 9}, { 8, 9,10}, {11,12,13}, {12,13,14}, {13,14,15}
	};

	public final static int MAX_BLOCKS = 15;
	public final static int BITS = 5;
	public final static int STARTING_PEG = 1;
	public final static int MAX_MOVES = MAX_BLOCKS - 1;

	ArrayList expression;
	ArrayList output = new ArrayList();

	String getBlock (int block, int moveNum){
		return "" + ((moveNum - 1) * MAX_BLOCKS + block) /*+ " (" + block + "," + moveNum + ")"*/;
 	}

	String getRule (int index, int moveNum){
		return "" + (MAX_MOVES * MAX_BLOCKS + index + 1 + (moveNum - 2) * legal.length) /*+ " [" + index + "," + moveNum + "]"*/;
	}

	void makeTransitions(){
		// Starting position.
		for (int a = 1; a <= MAX_BLOCKS; a++)
			if (a == STARTING_PEG) output.add ("-" + getBlock (a, 1) + " 0");
			else output.add (getBlock (a, 1) + " 0");

		// Ending position.
		for (int a = 1; a <= MAX_BLOCKS; a++)
			if (a == 1) output.add (getBlock (a, MAX_MOVES) + " 0");
			else output.add ("-" + getBlock (a, MAX_MOVES) + " 0");

		// Create in-between position transitions...
		for (int moves = 2; moves <= MAX_MOVES; moves++){

			boolean defined[] = new boolean[legal.length];
			boolean check[][] = new boolean[legal.length][legal.length];
			for (int a = 0; a < legal.length; a++) check[a][a] = true;

			// No change checking unless rule applied...
			for (int a = 1; a <= MAX_BLOCKS; a++){
				String previmpliescurr = "-" + getBlock(a, moves - 1) + " " + getBlock(a, moves);
				String currimpliesprev = "" + getBlock(a, moves - 1) + " -" + getBlock(a, moves);
				String changecond = "";
				for (int b = 0; b < legal.length; b++){
					if (a == legal[b][0] || a == legal[b][1] || a == legal[b][2]){
						
						// Define a new rule...
						if (!defined[b]){
							defined[b] = true;
							String ruleName = "-" + getRule (b, moves) + " ";
							output.add (ruleName + getBlock (legal[b][1], moves - 1) + " 0");
							output.add (ruleName + "-" + getBlock (legal[b][1], moves) + " 0");
							output.add (ruleName + getBlock (legal[b][0], moves - 1) + " " + getBlock (legal[b][2], moves - 1) + " 0");
							output.add (ruleName + "-" + getBlock (legal[b][0], moves - 1) + " -" + getBlock (legal[b][2], moves - 1) + " 0");
							output.add (ruleName + getBlock (legal[b][0], moves - 1) + " " + getBlock (legal[b][0], moves) + " 0");
							output.add (ruleName + "-" + getBlock (legal[b][0], moves - 1) + " -" + getBlock (legal[b][0], moves) + " 0");
							output.add (ruleName + getBlock (legal[b][2], moves - 1) + " " + getBlock (legal[b][2], moves) + " 0");
							output.add (ruleName + "-" + getBlock (legal[b][2], moves - 1) + " -" + getBlock (legal[b][2], moves) + " 0");
							
							
							// Make sure no other rules are being used.
							for (int c = 0; c < legal.length; c++){
								if (!check[b][c] && defined[c]){
									check[b][c] = check[c][b] = true;
									output.add ("-" + getRule (b, moves) + " -" + getRule (c, moves) + " 0");
								}
							}
							
						}
						changecond += " " + getRule (b, moves);
					}
				}
				output.add (previmpliescurr + changecond + " 0");
				output.add (currimpliesprev + changecond + " 0");
			}

			// Pick a rule.
			String t = "";
			for (int a = 0; a < legal.length; a++)
				t += getRule (a, moves) + " ";
			output.add (t + "0");

		}

		System.out.println ("p cnf " + getRule (legal.length - 1, MAX_MOVES) + " " + output.size());
		for (int a = 0; a < output.size(); a++)
			System.out.println (output.get (a));
	}

	void run(){
		makeTransitions();
	}

	public static void main (String args[]){
		(new MakePegTriangleRule()).run();
	}
}
